/*
 * Copyright (c) 1987,1997, Prentice Hall
 * All rights reserved.
 *
 * Redistribution and use of the MINIX operating system in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:

 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.

 * - Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials provided
 *   with the distribution.

 * - Neither the name of Prentice Hall nor the names of the software
 *   authors or contributors may be used to endorse or promote
 *   products derived from this software without specific prior
 *   written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS, AUTHORS, AND
 * CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PRENTICE HALL OR ANY AUTHORS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
/** @addtogroup FileSystemImpl
* @{
*/ 

/**
 * @file	block.c
 * @brief	Basic methods for reading blocks. 
 */

#include <stdio.h>
#include <async.h>
#include <sysinfo.h>
#include "fs.h"
#include "block.h"
#include "super.h"
#include "../rd/rd.h"

block_t* work_block; /**< A temporary variable for storing the working block */

/**
 * Allocates memory for working block
 */ 
int init_block()
{
	
	/* Allocating free space for working block. */
	work_block = (block_t*)malloc(sizeof(block_t));
	if (work_block != NULL)
		return TRUE;

	return FALSE;	
}

/**
 * Returns requested block according its number
 */
block_t *get_block(register block_num_t block_nr)
{
	
	if (block_nr == NO_BLOCK)
		return (get_zero_block());

	work_block->b_blocknr = block_nr;
	read_block(work_block);

	return work_block;
}

/**
 * Return a block filled with zeroes
 */ 
block_t *get_zero_block()
{

	/* Setting all bytes inside block to 0 */
	memset(work_block->b.b__data, 0, BLOCK_SIZE);

	return work_block;	
}

/**
 * Process reading of requested block
 */
void read_block(register block_t *bp)
{

	int r;
	offset_t pos, header_size;

	header_size = sysinfo_value("rd.header_size");
	pos = header_size + (offset_t)bp->b_blocknr * BLOCK_SIZE;
	
	/* Reading a block with specified number from ramdisk into shared buffer. */
	r = async_req_2(rd_phone, RD_READ_BLOCK, pos, BLOCK_SIZE, NULL, NULL); 
	if (r < 0) {
		print_console_int("Unrecoverable disk error on RAMDISK, block: %d\n", bp->b_blocknr);
		print_console_int("Error number: %d\n", r);
		status = FS_EIO;
		return;
	}
	
	/* Copy retrieved data from buffer into work block. */
	memcpy((void *)(bp->b.b__data), (void *)buffer, BLOCK_SIZE);
	status = BLOCK_SIZE;	
}


/**
 * }
 */
  
